# Machine learning example.
#
# Evaluation the performance of classification and clustering algorithms on a very difficult data set.
#
require(gdata)
require(RWeka)
require(rpart) 
require(party)
require(randomForest)

# read in the data 
vals <- read.xls("../data/data_set_2.xls", header=T)[,-1]
colnames(vals) <- c("Antigen", "Age", "Total", paste("c",1:16,sep=""))

# add class by age, 0=(0-40), 1=(41-60), 2=(61+)
vals <- cbind(vals,Class=as.factor(sapply(vals$Age,function(x) ifelse(x<=40,0,ifelse(x<=60,1,2)))))

# get rid of records with NA values in any column
vals <- na.omit(vals)

# standardize and center each column about 0
vals[,4:19] <- scale(vals[,4:19])

# split into the two experiment data sets, but we'll just focus on set 1 here
vals.exp1 <- vals[,c(1,2,4:9,20)]
vals.exp2 <- vals[,c(1,2,10:19,20)]

#--------------------------------------------------------------------------------------------------------------
# display summary information
summary(vals.exp1)

# let's do a quick pca and show the associated biplot
summary(pc.cr <- princomp(vals.exp1[,-c(1,2,9)], cor = TRUE))
biplot(pc.cr,xlabs=vals.exp1$Antigen,cex=c(0.5,1.0),col=c("gray50","red"))

#--------------------------------------------------------------------------------------------------------------
# Naive Bayesian
nb <- naiveBayes(vals.exp1[,-c(1,2,9)],vals.exp1[,9])
table(predict(nb,vals.exp1[,-c(1,2,9)]),vals.exp1[,9])

# -------------------------------------------------------------------------------------------------------------
# Plot the partitions of the classification methods
require(klaR)

pdf("../plots/lda.pdf")
partimat(Class~.,data=vals.exp1[,-c(1,2)],method="lda")
dev.off()

pdf("../plots/qda.pdf")
partimat(Class~.,data=vals.exp1[,-c(1,2)],method="qda")
dev.off()

pdf("../plots/rpart.pdf")
partimat(Class~.,data=vals.exp1[,-c(1,2)],method="rpart")
dev.off()

postscript("../plots/sknn.eps")
partimat(Class~.,data=vals.exp1[,-c(1,2)],method="sknn")
dev.off()

#--------------------------------------------------------------------------------------------------------------
# kNN classifier
#   find best k=1:20, using 10-fold cross-validation
fit.ibk <- IBk(Class ~., data=vals.exp1[,-c(1,2)], control = Weka_control(K = 20, X = TRUE))

evaluate_Weka_classifier(fit.ibk, numFolds = 10)

print(fit.ibk)

#--------------------------------------------------------------------------------------------------------------
# decision tree by recursive partitioning
#   also plot tree
fit.rpart <- rpart(Class ~., data=vals.exp1[,-c(1,2)], method="class",
                              control=rpart.control(minsplit=15,cp=0.001))
print(fit.rpart)

summary(fit.rpart)

# print complexity parameter
printcp(fit.rpart,digits=getOption("digits")-2)

# plot cross validation results versus cost-complexity (and tree size) 
#    best tree is ~where horizontal line crosses relative error
plotcp(fit.rpart)

# show the tree
plot(fit.rpart, compress=TRUE)
text(fit.rpart, use.n=TRUE)

# we can prune the tree down as well...
pfit <- prune(fit.rpart, cp=fit.rpart$cptable[which.min(fit.rpart$cptable[,"xerror"]),"CP"])
plot(pfit, uniform=TRUE, main="Pruned R-Part Tree")
text(pfit, use.n=TRUE, all=TRUE, cex=.8)

# can write to postscript if we want...
post(pfit, file = "../plots/pruned_rpart_tree.ps", title = "Pruned R-Part Tree")


#--------------------------------------------------------------------------------------------------------------
# random forest
#    display importance and plot MDS result visualization
fit.rf <- randomForest(Class~.,data=vals.exp1[,-c(1,2)],ntree=500, keep.forest=T, importance=T, proximity=T)
importance(ffit)
MDSplot(fit.rf, vals.exp1$Class)


###############################################################################################################
#                 CLUSTERING
###############################################################################################################

require(mclust)
model <- Mclust(vals.exp1[,-c(1,2,9)],G=3:9)  # minimum 3 clusters
plot(model,vals.exp1[,-c(1,2,9)])

# output to file too
pdf("../plots/mclust.pdf")
plot(model,vals.exp1[,-c(1,2,9)])
dev.off()

# confusion matrix
table(model$classification,vals.exp1$Class)


#--------------------------------------------------------------------------------------------------------------
# Multi-dimensional Scaling
# plot beside each other
par(mfrow=c(1,2),pty="s")

# metric mds
fit <- cmdscale(dist(vals.exp1[,-c(1,2,9)]),eig=TRUE, k=2)
plot(fit$points[,1],fit$points[,2],type="n",xlab="",ylab="",main="Metric MDS")
text(fit$points[,1],fit$points[,2],labels=vals.exp1$Class,cex=0.9)

#non-metric mds
fit <- isoMDS(dist(vals.exp1[,-c(1,2,9)]),k=2)
plot(fit$points[,1],fit$points[,2],type="n",xlab="",ylab="",main="Non-Metric MDS")
text(fit$points[,1],fit$points[,2],labels=vals.exp1$Class,cex=0.9)
