# Identify distribution of plaques
require(ggplot2)
require(gdata) 
require(MASS)
require(Matching)


# Function to plot multiple ggplot2 objects to the same output, IGNORE. 
multiplot <- function(..., plotlist=NULL, cols) {
   require(grid)

   # Make a list from the ... arguments and plotlist
   plots <- c(list(...), plotlist)

   numPlots = length(plots)

   # Make the panel
   plotCols = cols                          # Number of columns of plots
   plotRows = ceiling(numPlots/plotCols) # Number of rows needed, calculated from # of cols

   # Set up the page
   grid.newpage()
   pushViewport(viewport(layout = grid.layout(plotRows, plotCols)))
   vplayout <- function(x, y)
   viewport(layout.pos.row = x, layout.pos.col = y)

   # Make each plot, in the correct location
   for (i in 1:numPlots) {
      curRow = ceiling(i/plotCols)
      curCol = (i-1) %% plotCols + 1
      print(plots[[i]], vp = vplayout(curRow, curCol ))
   }
}

# Creates a customized QQ-plot of the given data to specific distributions
#
# data: the observed/sample data (as a data frame)
# coln: column in the data to consider
# dist: theoretical comparison distribution 
myplot.qq <- function( data, coln, 
                              dist=c("normal","cauchy","gamma","exponential","log-normal","weibull","logistic") ) {
   # we just want the specific column, without NAs.
   x      <- data[,coln]
   x      <- na.omit(x)
   colnm  <- colnames(data)[coln]
   
   # get the distribution asked for 
   dist   <- match.arg(dist)
   n.dist <- dist
   
   # ML fit and get params
   fit    <- fitdistr(x,dist)
   params <- as.list(fit$estimate)
   
   # map distribution names between those for fitting and for generating samples 
   # (fitdistr and the sampling algorithms use different names for some distributions)
   if ( dist=="normal" ) dist <- "norm"
   else if ( dist=="exponential" ) dist <- "exp"
   else if ( dist=="log-normal" ) dist <- "lnorm"
   else if ( dist=="logistic" ) dist <- "logis"
      
   # bootstrap ks-test (we may have ties, so a regular KS-test is not appropriate)
   ksb <- ks.boot(nboot=1000,x,do.call(paste("r",dist,sep=""),c(list(n=100*length(x)),params)))

   # do the plot! 
   rng <- c(-0.75,1.5)*range(c(x))
   p.str <- toString(paste(names(params),"=",round(unlist(params),2),sep=""))
   
   # lots of ggplot2 code! Most of it is formatting issues, actual work is: stat_qq, geom_abline for diag line
   q <- ggplot(data=vals, aes_string(sample=colnm), aes(size=2)) + 
                  stat_qq(distribution=paste("q",dist,sep=""),dparams=params) + 
                  opts(aspect.ratio=1,
                        axis.text.x=theme_text(size=12),
                        axis.text.y=theme_text(size=12),
                        axis.title.x=theme_text(face="bold", size=14),
                        axis.title.y=theme_text(face="bold", size=14,angle=90),
                        legend.title=theme_blank(),
                        legend.position="none",
                        legend.key=theme_rect(fill="white",colour="white"),
                        legend.background = theme_rect(),
                        panel.background = theme_blank(), 
                       title=colnm) + 
                  geom_abline(intercept=0,slope=1,col="gray40") + 
                  xlim(rng) + ylim(rng) + xlab(paste(n.dist,"  (",p.str,")",sep="")) +
                  ylab(paste("sample   (p-value=",round(mean(ksb$ks.boot.pvalue),2),")",sep=""))
   if ( length(params)==1 ) params <- c(params,NA)
   return(list(q,params))
}


# set plotting theme to black/white
theme_bw()

MIN_RADIUS <- 0.18

# read data from sheet 1 of excel file
vals <- read.xls("../data/data.xls", sheet=1, header=1)
vals <- data.frame(apply(vals,c(1,2),function(x) ifelse(x>MIN_RADIUS,sqrt(x/pi),NA))) 

# change colnames to strain identifier, long way for demo purposes
#1    PR8 REC
#3    BE89+5SY97
#19	BE92 REC
#20	BE92 N145K
#21	WU95 REC
#30	HK68 REC
ss <- colnames(vals)
ss <- gsub("strain_19.","BE92_REC_",ss)
ss <- gsub("strain_1.","PR8_REC_",ss)
ss <- gsub("strain_30.","HK68_REC_",ss)
ss <- gsub("strain_3.","BE89_5SY07_",ss)
ss <- gsub("strain_20.","BE92_N145K_",ss)
ss <- gsub("strain_21.","WU95_REC_",ss)
colnames(vals) <- ss

#--------------------------------------------------------------------------------------------------------------
# now we want to loop through all strains and compare each one to each distribution
cnt <- 1
params <- NULL
for ( s in ss ) {
   # do the QQ-plot
   p1 <- myplot.qq(vals,cnt,"normal")
   p2 <- myplot.qq(vals,cnt,"cauchy")
   p3 <- myplot.qq(vals,cnt,"gamma")
   p4 <- myplot.qq(vals,cnt,"exponential")
   p5 <- myplot.qq(vals,cnt,"log-normal")
   p6 <- myplot.qq(vals,cnt,"weibull")
   
   # track the mean for each distribution. for cauchy we use location as no mean exists.
   params <- rbind(params, cbind("normal",s,p1[[2]]$mean), cbind("cauchy",s,p2[[2]]$location), 
                   cbind("gamma",s,p3[[2]]$shape*p3[[2]]$rate^(-1)), cbind("exponential",s,p4[[2]]$rate^(-1)),
                   cbind("log-normal",s,exp(p5[[2]]$meanlog+p5[[2]]$sdlog^2/2)), 
                   cbind("weibull",s,p6[[2]]$scale*gamma(1+1/p6[[2]]$shape)))
      
   # plot all QQ-plots in 1 pdf
   pdf(paste("../plots/",s,".pdf",sep=""),width=25,height=5)   
   multiplot(p1[[1]], p2[[1]], p3[[1]], p4[[1]], p5[[1]], p6[[1]], cols=6)
   dev.off()
   
   cnt <- cnt + 1
}

#--------------------------------------------------------------------------------------------------------------
# split data into time intervals. We can use a linux grep-like command for this. 
n <- nrow(params)
df.30 <- data.frame(params[grep("30h",params[,2]),])
df.30[,3] <- as.numeric(params[ grep("30h",params[,2]),3])
df.35 <- data.frame(params[ grep("35h",params[,2]),])
df.35[,3] <- as.numeric(params[ grep("35h",params[,2]),3])
df.40 <- data.frame(params[ grep("40h",params[,2]),])
df.40[,3] <- as.numeric(params[ grep("40h",params[,2]),3])
df <- data.frame(params)
df[,3] <- as.numeric(params[,3])
colnames(df.30) <- c("Dist","Strain","Mean")
colnames(df.35) <- c("Dist","Strain","Mean")
colnames(df.40) <- c("Dist","Strain","Mean")
colnames(df) <- c("Dist","Strain","Mean")

#--------------------------------------------------------------------------------------------------------------
# dotplot the mean value of each distribution, per strain for 30h
p <- ggplot(data=df.30,aes(x=Mean,y=Dist)) + geom_point() + xlim(range(df[,3]))+ facet_wrap(~Strain,ncol=1) + 
         xlab("Mean/location of distribution") + ylab("Distribution")
ggsave(plot=p, filename="../plots/means30h.pdf",height=12,width=4,dpi=600)

#--------------------------------------------------------------------------------------------------------------
# dotplot the mean value of each distribution, per strain for 35h 
p <- ggplot(data=df.35,aes(x=Mean,y=Dist)) + geom_point() + xlim(range(df[,3]))+ facet_wrap(~Strain,ncol=1) + 
         xlab("Mean/location of distribution") + ylab("Distribution")
ggsave(plot=p, filename="../plots/means35h.pdf",height=12,width=4,dpi=600)

#--------------------------------------------------------------------------------------------------------------
# dotplot the mean value of each distribution, per strain for 40h
p <- ggplot(data=df.40,aes(x=Mean,y=Dist)) + geom_point() + xlim(range(df[,3]))+ facet_wrap(~Strain,ncol=1) + 
         xlab("Mean/location of distribution") + ylab("Distribution")
ggsave(plot=p, filename="../plots/means40h.pdf",height=12,width=4,dpi=600)


# Now add a time column
df <- cbind(df,Time=c(rep(30,6),rep(35,6),rep(40,6)))
df[,2] <- sapply(df[,2],function(x) gsub("_30h","",x))
df[,2] <- sapply(df[,2],function(x) gsub("_35h","",x))
df[,2] <- sapply(df[,2],function(x) gsub("_40h","",x))

#--------------------------------------------------------------------------------------------------------------
# plot all time points on single dotplot
p <- ggplot(data=df,aes(x=Mean,y=Dist, group=Time, shape=factor(Time))) + geom_point() + xlim(range(df[,3])) + 
         facet_wrap(~Strain,ncol=1) +
          xlab("Mean/location of distribution") + ylab("Distribution") +  scale_shape("Time (Hours)")
ggsave(plot=p,filename="../plots/meanAll.pdf",height=12,width=6,dpi=600)


#--------------------------------------------------------------------------------------------------------------
# plot change in mean over time, for each distribution 
p <- ggplot(data=df, aes(x=Time, y=Mean, group=Strain, shape=Strain)) + 
            geom_point() + stat_smooth(method="lm",se=F,col="gray20",linetype=2) + facet_wrap(~Dist)
ggsave(plot=p,filename="../plots/growthByDist.pdf",height=12,width=14,dpi=600)


#--------------------------------------------------------------------------------------------------------------
# linear model of mean over time, and confidence interval on growth rate plot
tmp <- with(df, by(df, df[,c("Strain","Dist")],function(x) lm(Mean ~ Time, data = x)))
cf <- sapply(tmp,coef)
ci <- sapply(tmp,confint,"Time")

df2 <- data.frame(t(cf),t(ci))
colnames(df2) <- c(colnames(df2)[1:2],"ciL","ciU")
df2 <- data.frame(df2,Strain=levels(factor(df$Strain)),Dist=rep(levels(df$Dist),each=6))

# plot growth rates along with error bars
p <- ggplot(data=df2, aes(x=Strain,y=Time)) + geom_point() + facet_wrap(~Dist) + 
            geom_errorbar(aes(ymin=ciL,ymax=ciU,width=0.15)) + 
            opts(axis.text.x=theme_text(angle=90,hjust=1)) + ylab("Slope (estimated growth rate)")
ggsave(plot=p,filename="../plots/ciGrowthRate.pdf",width=12,height=9)

